/* Example 08: Wake on LAN */

#include <WiFi.h>
#include <WiFiUdp.h>

// [Settings]
// ------------------------------------------------------------------------------------------------------
const char * networkName PROGMEM = "your_ssid";                          // AP の SSID
const char * networkPswd PROGMEM = "your_password";                      // AP のパスワード
const char * broadcastIP PROGMEM = "192.168.0.255";                      // ブロードキャストアドレス
const byte wolMAC[6] PROGMEM     = {0x01, 0x02, 0x03, 0x04, 0x05, 0xD6}; // WOL ターゲットの MAC ADDRESS
const int ledPin PROGMEM         = 22;                                   // LED のピン (IO22)
const int btnPin PROGMEM         = 23;                                   // ボタンのピン (IO23)
// ------------------------------------------------------------------------------------------------------

const byte preamble[6] PROGMEM = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};   // マジックパケットのヘッダ
const int wolPort PROGMEM = 7;                                           // マジックパケット送出ポート 

boolean connected = false;
WiFiUDP udp;

// セットアップ
void setup() {
  Serial.begin(115200);                                // シリアルポートを開く
  while(!Serial);                                      // シリアルポートが有効になるまで待つ
  
  pinMode(ledPin, OUTPUT);                             // LED のピンを出力に設定
  pinMode(btnPin, INPUT_PULLUP);                       // ボタンのピンを出力に設定し、内部プルアップをオンにする
  digitalWrite(ledPin, LOW);                           // とりあえず LED は消灯
  
  connectToWiFi(networkName, networkPswd);             // 指定された SSID とパスワードで Wi-Fi に接続
}

// ループ
void loop() {
  if (connected && (digitalRead(btnPin) == LOW)) {     // Wi-Fi に接続されている状態でボタンが押されたら
    sendMagicPacket();                                 // マジックパケットを送出する
    delay(2000);                                       // チャタリング / ボタン連打防止のため 2 秒待つ
  }
}

// Wi-Fi 接続
void connectToWiFi(const char * ssid, const char * pwd) {
  Serial.println("Connecting to WiFi network: " + String(ssid));
  WiFi.disconnect(true);                               // Wi-Fi から切断
  WiFi.onEvent(WiFiEvent);                             // Wi-Fi イベントハンドラを登録
  WiFi.begin(ssid, pwd);                               // 指定された SSID とパスワードで Wi-Fi に接続
  Serial.println("Waiting for WIFI connection...");
}

// Wi-Fi イベントハンドラ
void WiFiEvent(WiFiEvent_t event) {
  switch(event) {
    // [イベント] Wi-Fi 経由で DHCP から IP アドレスが振られた
    case SYSTEM_EVENT_STA_GOT_IP:
      Serial.print("WiFi connected! IP address: ");
      Serial.println(WiFi.localIP());                  // 取得した IP アドレスをシリアルに送信
      udp.begin(WiFi.localIP(), wolPort);              // UDP 通信を開始
      digitalWrite(ledPin, HIGH);                      // インジケーター用 LED を点灯
      connected = true;                                // Wi-Fi 接続フラグを true に
      break;
    // [イベント] Wi-Fi が切断された
    case SYSTEM_EVENT_STA_DISCONNECTED:
      Serial.println("WiFi lost connection");          
      digitalWrite(ledPin, LOW);                       // インジケーター用 LED を消灯
      connected = false;                               // Wi-Fi 接続フラグを false に
      break;
    // [イベント] その他
    default:
      break;                                           // 何もしない
  }
}

// マジックパケットの送出
void sendMagicPacket() {
  Serial.println("Wake up!");      
  udp.beginPacket(broadcastIP, wolPort);               // パケットの開始。ブロードキャストアドレスとポートを指定。
  udp.write(preamble, sizeof preamble);                // マジックパケットのヘッダを準備
  for (byte i=0; i<16; i++)                            // 受信する PC の MAC アドレスを 16 回 
    udp.write(wolMAC, sizeof wolMAC);                  //   繰り返したものを準備
  udp.endPacket();                                     // パケットの終了 (パケットの送信)
}

