/* Example 09: Simple Wi-Fi Server (AP) */

#include "WiFi.h"
#include <WiFiClient.h>

// [Settings]
// ------------------------------------------------------------------------------------------------------
const char *ssid = "ESP32_AP";                   // AP の SSID
const char *password = "12345678";               // AP のパスワード

const IPAddress local_ip(192, 168, 100, 1);      // IP アドレス
const IPAddress gateway(192, 168, 100, 1);       // デフォルトゲートウェイ
const IPAddress subnet(255, 255, 255, 0);        // サブネットマスク
// ------------------------------------------------------------------------------------------------------

WiFiServer server(80);

void setup() {
  Serial.begin(115200);
  pinMode(5, OUTPUT);      
  delay(10);

  Serial.println();
  Serial.println("Configuring access point...");
  WiFi.disconnect(true);                         // Wi-Fi を切断
  WiFi.mode(WIFI_MODE_AP);                       // Wi-Fi を AP モードに
  delay(10);
  WiFi.softAP(ssid, password);                   // AP の SSID とパスワードを設定
  delay(10);
  WiFi.softAPConfig(local_ip, gateway, subnet);  // AP の IP アドレス等を設定
  delay(10);

  IPAddress myIP = WiFi.softAPIP();              // 設定された AP の IP アドレスを取得
  Serial.print("AP IP address: ");
  Serial.println(myIP);                          // IP アドレスをシリアルに出力
  Serial.println();

  Serial.println("HTTP server started");
  server.begin();                                // HTTP サーバを開始
}

void loop() {
  WiFiClient client = server.available();   // listen for incoming clients

  if (client) {                             // if you get a client,
    Serial.println("New Client.");           // print a message out the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    while (client.connected()) {            // loop while the client's connected
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        if (c == '\n') {                    // if the byte is a newline character

          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println();

            // the content of the HTTP response follows the header:
            client.print("Click <a href=\"/H\">here</a> to turn the LED on pin 5 on.<br>");
            client.print("Click <a href=\"/L\">here</a> to turn the LED on pin 5 off.<br>");

            // The HTTP response ends with another blank line:
            client.println();
            // break out of the while loop:
            break;
          } else {    // if you got a newline, then clear currentLine:
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }

        // Check to see if the client request was "GET /H" or "GET /L":
        if (currentLine.endsWith("GET /H")) {
          digitalWrite(5, HIGH);               // GET /H turns the LED on
        }
        if (currentLine.endsWith("GET /L")) {
          digitalWrite(5, LOW);                // GET /L turns the LED off
        }
      }
    }
    // close the connection:
    client.stop();
    Serial.println("Client Disconnected.");
  }
}

